use serde::Deserialize;
use serde::Serialize;

use super::super::imprint::Imprint;
use super::Work;
use super::WorkStatusDefinition;
use super::WorkTypeDefinition;

pub const WORK_QUERY: &str = "
    query WorkQuery($workId: Uuid!, $publishers: [Uuid!]) {
        work(workId: $workId) {
            workId
            workType
            workStatus
            fullTitle
            title
            subtitle
            reference
            edition
            doi
            publicationDate
            place
            width
            height
            pageCount
            pageBreakdown
            imageCount
            tableCount
            videoCount
            license
            copyrightHolder
            landingPage
            lccn
            oclc
            shortAbstract
            longAbstract
            generalNote
            toc
            coverUrl
            coverCaption
            contributions {
                workId
                contributorId
                contributionType
                mainContribution
                biography
                institution
                lastName
                fullName
                contributor {
                    contributorId
                    lastName
                    fullName
                }
            }
            publications {
                publicationId
                publicationType
                workId
                isbn
                publicationUrl
                prices {
                    priceId
                    publicationId
                    currencyCode
                    unitPrice
                }
                work {
                    imprint {
                        publisher {
                            publisherId
                        }
                    }
                }
            }
            languages {
                languageId
                workId
                languageCode
                languageRelation
                mainLanguage
            }
            fundings {
                fundingId
                workId
                funderId
                program
                projectName
                projectShortname
                grantNumber
                jurisdiction
                funder {
                    funderId
                    funderName
                }
            }
            subjects {
                subjectId
                workId
                subjectType
                subjectCode
                subjectOrdinal
            }
            issues {
                workId
                seriesId
                issueOrdinal
                series {
                    seriesId
                    seriesType
                    seriesName
                    issnPrint
                    issnDigital
                    seriesUrl
                    imprint {
                        imprintId
                        imprintName
                        publisher {
                            publisherId
                            publisherName
                            publisherShortname
                            publisherUrl
                        }
                    }
                }
            }
            imprint {
                imprintId
                imprintName
                publisher {
                    publisherId
                    publisherName
                    publisherShortname
                    publisherUrl
                }
            }
        }
        imprints(limit: 9999, publishers: $publishers) {
            imprintId
            imprintName
            publisher {
                publisherId
                publisherName
                publisherShortname
                publisherUrl
            }
        }
        work_types: __type(name: \"WorkType\") {
            enumValues {
                name
            }
        }
        work_statuses: __type(name: \"WorkStatus\") {
            enumValues {
                name
            }
        }
    }
";

graphql_query_builder! {
    WorkRequest,
    WorkRequestBody,
    Variables,
    WORK_QUERY,
    WorkResponseBody,
    WorkResponseData,
    FetchWork,
    FetchActionWork
}

#[derive(Debug, Clone, Default, Serialize, Deserialize, PartialEq)]
#[serde(rename_all = "camelCase")]
pub struct Variables {
    pub work_id: Option<String>,
    pub publishers: Option<Vec<String>>,
}

#[derive(Debug, Clone, Default, Serialize, Deserialize, PartialEq)]
pub struct WorkResponseData {
    pub work: Option<Work>,
    pub imprints: Vec<Imprint>,
    pub work_types: WorkTypeDefinition,
    pub work_statuses: WorkStatusDefinition,
}
